import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

public class DnDTest extends Frame implements DropTargetListener {

  DropTarget fTarget;
  TextArea fTextArea;

  public DnDTest () {
    fTextArea = new TextArea("Grab some text and \npull it over here!");
    fTarget = new DropTarget(fTextArea, DnDConstants.ACTION_COPY, this);
    fTarget.setActive (true);
    add ("Center", fTextArea);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
  }

  public static void main (String args[]) {
    DnDTest theFrame = new DnDTest();
    theFrame.setSize (400, 200);
    theFrame.setVisible (true);
  }

// required methods in DropTargetListener

  public  void dragEnter (DropTargetDragEvent dtde) {
    System.out.println ("dragEnter");

    // use the DropTargetDragEvent to find out what flavors are offered
    DataFlavor df[] = dtde.getCurrentDataFlavors();
    for (int i = 0; i < df.length; i++)   {
      if (df[i].isMimeTypeEqual ("text/plain")) {

        // if the flavor is plain text, accept the drag
        dtde.acceptDrag (DnDConstants.ACTION_COPY);
        return;
      }
    }
    // otherwise reject this drag
    dtde.rejectDrag ();
  }

  public  void dragOver (DropTargetDragEvent dtde) {

    // dragOver is called repeatedly while the cursor is over
    // the target. Uncomment the println to see how often this
    // method is called.
    //System.out.println ("dragOver");
  }

  public void dropActionChanged(DropTargetDragEvent dtde) {
    System.out.println("dropActionChanged");
  }

  public  void dragScroll (DropTargetDragEvent dtde) {
    System.out.println ("dragScroll");
  }

  public  void dragExit (DropTargetEvent dte) {
    System.out.println ("dragExit");
  }

  public  void drop (DropTargetDropEvent dtde) {
    dtde.acceptDrop (DnDConstants.ACTION_COPY);
    System.out.println ("dropped");
    Transferable theTransferable = dtde.getTransferable();
    DataFlavor df[] = dtde.getCurrentDataFlavors();
    Object theObject = null;

    try {

      // search through the flavor looking for plain text
      // when you find it get the associated data and stuff
      // it into the Object.
      for (int i = 0; i < df.length; i++)   {
        if (df[i].isMimeTypeEqual ("text/plain")) {
          theObject = theTransferable.getTransferData(df[i]);
        }
      }

      // if the data comes from outside the JVM, it comes in
      // on an InputStream. Read in the stream.
      if (theObject != null && theObject instanceof InputStream) {
        InputStream theInputStream = (InputStream) theObject;
        StringBuffer theStringBuffer = new StringBuffer();
        byte[] aBuffer = new byte[64];
        int theCount = theInputStream.read(aBuffer);
        while (theCount != -1) {
          // keep stuffing data into the string buffer
          theStringBuffer.append (new String (aBuffer, 0, theCount));
          theCount = theInputStream.read(aBuffer);
        }
        // We've read in all the data. Now let's use it
        theInputStream.close();
        fTextArea.setText (theStringBuffer.toString());
      }
    } catch (Exception e) {
    e.printStackTrace();
    } finally {
      try {
        fTarget.getDropTargetContext().dropComplete(true);
      } catch (Exception ignore) {}
    }
  }

// let's handle a click on the window's close box
  protected void processWindowEvent (WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
      System.exit(0);
    super.processWindowEvent (e);
  }
}
