import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.awt.image.ImageObserver;
import java.awt.Rectangle;

public class TImage extends java.applet.Applet
{
  String fImageName = "";
  boolean fScalingComplete = false;
  Image fImage = null;
  Color fFrameColor = null;
  Color fMatteColor = null;
  Rectangle fBounds = null;
  int SCALE_FACTOR = 100;
  int FRAME_THICKNESS = 10;
  int MATTE_THICKNESS = 30;

  public void init()
  {
    setFrame("#000000");
    setMatte("#FFFFFF");
    setImage("logo.jpg");
  }

  public void setFrame(String theColor)
  {
    fFrameColor = getColorFromHex(theColor);
    repaint();
  }

  public void setMatte(String theColor)
  {
    fMatteColor = getColorFromHex(theColor);
    repaint();
  }

  public void setImage(String theImageName)
  {
    //System.out.println("setting image to " + theImageName + ".");
    fImageName = theImageName;
    fScalingComplete = false;

    //System.out.println("in setImage, codebase is " + getCodeBase());
    //System.out.println("in setImage, fImageName is " + fImageName + ".");
    if (fImageName != null && fImageName.length() != 0)
    {
      //System.out.println("About to get original image: " + getCodeBase() + ":" + fImageName + ".");
      fImage = getImage(getCodeBase(), fImageName);
      if (fImage != null)
        ;//System.out.println("Got " + fImage.toString());
      else
        System.out.println("getImage failed");
    }
    else 
      System.out.println("in setImage, fImageName is null or has zero length");
    if (fImage != null)
    {
      //System.out.println("in setImage, preparing to scaleDown");
      Scaler theScaler = new Scaler();
      int theWidth = fImage.getWidth(theScaler);
      int theHeight = fImage.getHeight(theScaler);

      //System.out.println("Image is " + theWidth + " x " + theHeight + ".");
      fBounds = getBounds();
      //System.out.println("Bounds are " + fBounds.width + " x " + fBounds.height + ".");

      if (theWidth != -1 && theHeight != -1)
      {
        fImage = setImage(new Rectangle(theWidth, theHeight));
        //System.out.println("painting synchronously");
        repaint();
      }
      // else control passes to imageUpdate;
    }
    else
      System.out.println("in setImage, theImage is null");

  }

  public void paint(Graphics g)
  {
    //System.out.println("Painting . . .");

    // don't allow the full-size image to paint
    if (fImage != null && fScalingComplete)
    {      
      //System.out.println(" the image " + fImage.toString());
      int theImageWidth = fImage.getWidth(this);
      int theImageHeight = fImage.getHeight(this);

      // center the image
      int theXCoord = fBounds.width/2 - theImageWidth/2;
      int theYCoord = fBounds.height/2 - theImageHeight/2;

      // draw the frame

      // three-D rectangles usually look best against a matching background
      setBackground(fFrameColor.brighter());
      g.setColor(fFrameColor);
      g.fill3DRect(theXCoord - MATTE_THICKNESS - FRAME_THICKNESS,
                   theYCoord - MATTE_THICKNESS - FRAME_THICKNESS,
                   theImageWidth + 2*MATTE_THICKNESS + 2*FRAME_THICKNESS,
                   theImageHeight + 2*MATTE_THICKNESS + 2*FRAME_THICKNESS,
                   true);

      // draw the matte
      g.setColor(fMatteColor);
      g.fillRect(theXCoord - MATTE_THICKNESS,
                 theYCoord - MATTE_THICKNESS,
                 theImageWidth + 2*MATTE_THICKNESS,
                 theImageHeight + 2*MATTE_THICKNESS);

      g.drawImage(fImage, theXCoord, theYCoord, this);
    }
  }

  // compute a rectangle that maps a large rectangle (such as an image) 
  // into a small rectangle (such as the clip rectangle of a Graphics object
  private Rectangle scaleDown(Rectangle theLargeRectangle, Rectangle theSmallRectangle)
  {
    Rectangle theResult = null;
    int theNewHeight = Integer.MAX_VALUE;
    int theNewWidth = Integer.MAX_VALUE;
    int theRatio = Integer.MAX_VALUE;
    if ((theSmallRectangle.width > theLargeRectangle.width) &&
        (theSmallRectangle.height > theLargeRectangle.height))
    {
      //System.out.println("In scaleDown, no scaling required.");
      theResult = theLargeRectangle; // it fits!
    }
    else
    {
      // scale the height; see if the width fits
      //System.out.println("Scaling height: " + theSmallRectangle.height + ":" +
      //                                        theLargeRectangle.height + ".");
      theRatio = (SCALE_FACTOR  * theSmallRectangle.height) / theLargeRectangle.height;
      //System.out.println("Ratio is " + theRatio + ".");

      // use fixed-point with a scaling factor
      if ( theRatio < SCALE_FACTOR )
      {
        //System.out.println("Scaling down by height; ratio is 1:" + theRatio);
        theNewHeight = theSmallRectangle.height;
        theNewWidth = (theLargeRectangle.width * theRatio)/SCALE_FACTOR;
      }
      if (theNewWidth > theSmallRectangle.width)
      {
        //System.out.println("Scaling width: " + theSmallRectangle.width + ":" +
        //                                       theLargeRectangle.width + ".");
        theRatio = (SCALE_FACTOR * theSmallRectangle.width) / theLargeRectangle.width;
        //System.out.println("Scaling down by width; ratio is 1:" + theRatio);
        theNewWidth = theSmallRectangle.width;
        theNewHeight = (theLargeRectangle.height * theRatio)/SCALE_FACTOR;
      }
      theResult = new Rectangle(theNewWidth, theNewHeight);
    }
    return theResult;
  }


  private Image setImage(Rectangle theImageRectangle)
  {
    Image theResult = null;
    //System.out.println("Calling setImage with width: " + theImageRectangle.width + 
    //                   ", height: " + theImageRectangle.height);
    //System.out.println("bounds wd:" + fBounds.width + " ht:" + fBounds.height + ".");
    Rectangle theNewRectangle = scaleDown(theImageRectangle, fBounds);
    //System.out.println("in setImage(rect), scaled down to " + theNewRectangle.width + " x " +
    //                    theNewRectangle.height + ".");
    //System.out.println("setImage(r) about to scale down " + fImage.toString());
    theResult = fImage.getScaledInstance(theNewRectangle.width,
                                         theNewRectangle.height,
                                         Image.SCALE_DEFAULT);
    //System.out.println("The scaled down image is " + theResult.toString());
    fScalingComplete = true;
    return theResult;
  }

  private Color getColorFromHex(String theColor)
  {
    Color theResult = null;

    // skip the leading '#'
    String theRed = theColor.substring(1,3);
    String theGreen = theColor.substring(3,5);
    String theBlue = theColor.substring(5,7);
    //System.out.println("getColorFromHex has R: " + theRed + " G: " + theGreen + " B: " + theBlue + ".");
    int r = Integer.parseInt(theRed, 16);
    int g = Integer.parseInt(theGreen, 16);
    int b = Integer.parseInt(theBlue, 16);
    theResult = new Color(r, g, b);
    return theResult;
  }

  private class Scaler implements ImageObserver
  {
    public boolean imageUpdate(Image theImage, int theInfoFlags, int x, int y, int width, int height)
    {
      boolean theResult = false; // don't need more info
      //System.out.println("Someone called Scaler.imageUpdate; width is " + 
      //                    width + " height is " + height + ".");
      if (((theInfoFlags & ImageObserver.ABORT) == ImageObserver.ABORT) ||
          ((theInfoFlags & ImageObserver.ERROR) == ImageObserver.ERROR))
        theResult = false; // bail out
      else 
        if ((theInfoFlags & ImageObserver.ALLBITS) == ImageObserver.ALLBITS)
        {
          fImage = setImage(new Rectangle(width, height));
          //System.out.println("About to paint asynchronously");
          repaint();
        } else
        {
          //System.out.println("Waiting for image width and height.");
          theResult = true;
        }
      return theResult;
    }
  }
}
